﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Web;
using Infragistics.Web.Mvc;

namespace WebMonitor.Models
{
    public static class CustomGridModel
    {
        //This could later become a builder or factory... But I think that we want it to just return a new gridModel with all
        // of the correct settings

        public static GridModel BuildCustomGridModel(GridModel gridModel)
        {

            AddFeatures(gridModel);

            SetColumns(gridModel);

            return gridModel;
        }

        //This adds all of the features to the infragistics grid
        private static void AddFeatures(GridModel gridModel)
        {
            GridSelection gridSelection = new GridSelection
            {
                
                MultipleSelection = true,
                //Activation = false,
                Mode = SelectionMode.Row
            };
            gridModel.Features.Add(gridSelection); //Allows the user to select items

            gridModel.Features.Add(new GridSorting()); //Allows the user to sort by column

            gridModel.Features.Add(new GridFiltering()); // Allows the user to filter by column

            //GridRowSelectors gridRowSelectors = new GridRowSelectors();
            //gridModel.Features.Add(gridRowSelectors); // Allows the user to select rows

            gridModel.Features.Add(new GridColumnMoving()); // Allows the user to drag and drop columns

            GridUpdating gridUpdating = new GridUpdating
            {
                EnableAddRow = false,
                EnableDeleteRow = false,
                EditMode = GridEditMode.None
            };
            gridModel.Features.Add(gridUpdating); //Allows the grid to be updated Used to update on the fly with JSP

            gridModel.Features.Add(new GridGroupBy()); //Allows users to group by columns

            gridModel.AutoCommit = true; // Makes sure all data changes are loaded in a single transaction
            //gridModel.Features.Add(new GridHiding()); //allows columns to be hidden and shown if they are already hidden
        }

        // Columns cannot be hidden such that the user doesn't have access to them. 
        // The key must match the datatype found in the datasource: in this case the MasterJobRow class
        private static void SetColumns(GridModel gridModel)
        {
            gridModel.AutoGenerateColumns = false;

            GridColumn jobIdColumn = new GridColumn();
            jobIdColumn.Key = "JobID";
            jobIdColumn.HeaderText = "Job ID";
            jobIdColumn.DataType = "int";
            gridModel.Columns.Add(jobIdColumn);

            GridColumn historyIdColumn = new GridColumn();
            historyIdColumn.Key = "JobHistoryID";
            historyIdColumn.HeaderText = "History ID";
            historyIdColumn.DataType = "int";
            gridModel.Columns.Add(historyIdColumn);

            GridColumn primaryCategoryColumn = new GridColumn();
            primaryCategoryColumn.Key = "PrimaryCategory";
            primaryCategoryColumn.HeaderText = "Primary Category";
            primaryCategoryColumn.DataType = "string";
            gridModel.Columns.Add(primaryCategoryColumn);

            GridColumn downstreamDependentJobsColumn = new GridColumn();
            downstreamDependentJobsColumn.Key = "DownstreamDependentJobs";
            downstreamDependentJobsColumn.HeaderText = "Downstream Dependent Jobs";
            downstreamDependentJobsColumn.DataType = "List<int>";
            gridModel.Columns.Add(downstreamDependentJobsColumn);

            GridColumn upstreamDependentJobsColumn = new GridColumn();
            upstreamDependentJobsColumn.Key = "UpstreamDependentJobsColumn";
            upstreamDependentJobsColumn.HeaderText = "Upstream Dependent Jobs";
            upstreamDependentJobsColumn.DataType = "List<int>";
            gridModel.Columns.Add(upstreamDependentJobsColumn);

            GridColumn jobNameColumn = new GridColumn();
            jobNameColumn.Key = "JobName";
            jobNameColumn.HeaderText = "Job Name";
            jobNameColumn.DataType = "string";
            gridModel.Columns.Add(jobNameColumn);

            GridColumn machineColumn = new GridColumn();
            machineColumn.Key = "Machine";
            machineColumn.HeaderText = "Machine";
            machineColumn.DataType = "string";
            gridModel.Columns.Add(machineColumn);

            GridColumn statusColumn = new GridColumn();
            statusColumn.Key = "Status";
            statusColumn.HeaderText = "Status";
            statusColumn.DataType = "string";
            statusColumn.Hidden = true;             //TODO: Is currently hidden
            gridModel.Columns.Add(statusColumn);

            GridColumn startTimeColumn = new GridColumn();
            startTimeColumn.Key = "StartTime";
            startTimeColumn.HeaderText = "Start Time";
            startTimeColumn.DataType = "dateTime";
            startTimeColumn.Format = "dateTime";
            gridModel.Columns.Add(startTimeColumn);

            GridColumn endTimeColumn = new GridColumn();
            endTimeColumn.Key = "EndTime";
            endTimeColumn.HeaderText = "End Time";
            endTimeColumn.DataType = "dateTime";
            endTimeColumn.Format = "dateTime";
            gridModel.Columns.Add(endTimeColumn);

            GridColumn elapsedTimeColumn = new GridColumn();
            elapsedTimeColumn.Key = "ElapsedTime";
            elapsedTimeColumn.HeaderText = "Elapsed Time";
            elapsedTimeColumn.DataType = "dateTime";
            elapsedTimeColumn.Format = "dateTime";
            gridModel.Columns.Add(elapsedTimeColumn);

            GridColumn userColumn = new GridColumn();
            userColumn.Key = "User";
            userColumn.HeaderText = userColumn.Key;
            userColumn.DataType = "string";
            gridModel.Columns.Add(userColumn);

            GridColumn ITSupport1Column = new GridColumn();
            ITSupport1Column.Key = "ITSupport1";
            ITSupport1Column.HeaderText = "IT Support1";
            ITSupport1Column.DataType = "string";
            gridModel.Columns.Add(ITSupport1Column);

            GridColumn ITSupport2Column = new GridColumn();
            ITSupport2Column.Key = "ITSupport2";
            ITSupport2Column.HeaderText = "IT Support2";
            ITSupport2Column.DataType = "string";
            gridModel.Columns.Add(ITSupport2Column);

            GridColumn SchedulerTypeColumn = new GridColumn();
            SchedulerTypeColumn.Key = "SchedulerType";
            SchedulerTypeColumn.HeaderText = "Scheduler Type";
            SchedulerTypeColumn.DataType = "string";
            gridModel.Columns.Add(SchedulerTypeColumn);

            GridColumn AverageDurationColumn = new GridColumn();
            AverageDurationColumn.Key = "AverageDurationInMinutes";
            AverageDurationColumn.HeaderText = "Average Duration In Minutes";
            AverageDurationColumn.DataType = "Decimal";
            gridModel.Columns.Add(AverageDurationColumn);

            GridColumn StandardDeviationOfDurationColumn = new GridColumn();
            StandardDeviationOfDurationColumn.Key = "StandardDeviationOfDurationInMinutes";
            StandardDeviationOfDurationColumn.HeaderText = "Standard Deviation Of Duration In Minutes";
            StandardDeviationOfDurationColumn.DataType = "Decimal";
            gridModel.Columns.Add(StandardDeviationOfDurationColumn);

#region Hidden Columns
            GridColumn jobStatusIdColumn = new GridColumn();  //TODO not currently hidden
            jobStatusIdColumn.Key = "JobStatusID";
            jobStatusIdColumn.HeaderText = "Job Status ID";
            jobStatusIdColumn.DataType = "int";
            gridModel.Columns.Add(jobStatusIdColumn);

            GridColumn jobStatusColorColumn = new GridColumn();
            jobStatusColorColumn.Key = "StatusColor";
            jobStatusColorColumn.Hidden = true;
            jobStatusColorColumn.DataType = "string";
            gridModel.Columns.Add(jobStatusColorColumn);

            GridColumn jobSchedulerIdColumn = new GridColumn();
            jobSchedulerIdColumn.Key = "JobSchedulerId";
            jobSchedulerIdColumn.Hidden = true;
            jobSchedulerIdColumn.DataType = "int";
            gridModel.Columns.Add(jobSchedulerIdColumn);

            GridColumn typeColumn = new GridColumn();
            typeColumn.Key = "Type";
            typeColumn.Hidden = true;
            typeColumn.DataType = "int";
            gridModel.Columns.Add(typeColumn);

            GridColumn allowRerunColumn = new GridColumn();
            allowRerunColumn.Key = "AllowRerun";
            allowRerunColumn.Hidden = true;
            allowRerunColumn.DataType = "int";
            gridModel.Columns.Add(allowRerunColumn);

            GridColumn allowUnconditionalRerunColumn = new GridColumn();
            allowUnconditionalRerunColumn.Key = "AllowUnconditionalRerun";
            allowUnconditionalRerunColumn.Hidden = true;
            allowUnconditionalRerunColumn.DataType = "int";
            gridModel.Columns.Add(allowUnconditionalRerunColumn);

            GridColumn AllowCancelColumn = new GridColumn();
            AllowCancelColumn.Key = "AllowCancel";
            AllowCancelColumn.Hidden = true;
            AllowCancelColumn.DataType = "int";
            gridModel.Columns.Add(AllowCancelColumn);

            #endregion

        }
    }
}